/************************************************************************************************************
Week planner
Copyright (C) Januar 2006 - May 2011  DTHMLGoodies.com, Alf Magne Kalleland

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

Dhtmlgoodies.com., hereby disclaims all copyright interest in this script
written by Alf Magne Kalleland.

Alf Magne Kalleland, 2011
Owner of DHTMLgoodies.com

************************************************************************************************************/



/* User variables */
var headerDateFormat = 'd.m';	// Format of day, month in header, i.e. at the right of days
var instantSave = true;	// Save items to server every time something has been changed (i.e. moved, resized or text changed) - NB! New items are not saved until a description has been written?

var remote = {
    urls : {
        'get'       : 'weekplanner-controller.php',     // File called by ajax when changes are loaded from the server(by Ajax).
        'save'      : 'weekplanner-controller.php',         // File called by ajax when changes are made to an element
        'delete'    : 'weekplanner-controller.php',       // File called by ajax when an element is deleted. Input to this file is the variable "eventToDeleteId=<id>"
        'edit'      : 'edit_event.php'                  // Called when double clicking on an event. use false if this option is disabled.
    },
    refreshViewAfterSeconds : 0,                       // Automatically reload view from server, set to 0 to disable
    method : 'post',
    useJSON : true
};

var txt = {
    'delete' : 'Click OK to delete this event',  // Text in dialog box - confirm before deleting an event
    'holdDownToEdit' : 'Press down mouse button for 1 second to edit',
    'loadError' : ''    // When defined, this txt will be alerted when the script is unable to load data from the server
}

var appointmentMarginSize = 5;	// Margin at the left and right of appointments;
var initialScrollToHour = 8;	// Initially auto scroll scheduler to the position of this hour
var initMinutes = 15;	// Used to auto set start time. Example: 15 = auto set start time to 0,15,30 or 45. It all depends on where the mouse is located ondragstart
var snapToMinutes = 15;	// Snap to minutes, example: 5 = allow minute 0,5,10,15,20,25,30,35,40,45,50,55
var weekplannerStartHour=0;	// If you don't want to display all hours from 0, but start later, example: 8am

var inlineTextAreaEnabled = true;	// Edit events from inline textarea?


/* End user variables */



if(!window.DG) {
	window.DG = {};
};




DG.WeekPlannerModel = new Class( {
    Extends: Events,
    data : [],
    recordPointers : {},
    counterNewIds : 10000,

    initialize : function() {

    },

    clear : function() {
        this.data = [];
        this.recordPointers = {};
    },

    addRecord : function(record) {
        if(!record.id){
            record.id = this.getNewId();
        }
        this.data.push(record);
        this.recordPointers[record.id] = this.data.length-1;
       // this.fireEvent('add', record);
    },
    getRecord : function(id){
        var index = this.getRecordIndex({ id : id} );
        return this.data[index];
    },
    update : function(record){
        var recordIndex = this.getRecordIndex(record);
        if(recordIndex == -1){
            this.addRecord(record);
        }else{
            var storeRecord = this.data[recordIndex];
            if(storeRecord.id != record.id){
                this.recordPointers[record.id] = recordIndex;
            }
            storeRecord = Object.merge(storeRecord, record);
        }
        this.fireEvent('update', record);
    },
    getRecordIndex : function(record){
        if(this.recordPointers[record.id] !== undefined){
            return this.recordPointers[record.id];
        }else{
            var id = this.getNumeric(record.id);
            if(this.recordPointers[id] !== undefined){
                return this.recordPointers[id];
            }
            return -1;
        }
    },
    deleteRecord : function(record){
        var index = this.getRecordIndex(record);
        this.data[index] = null;
        this.recordPointers[record.id] = null;
        this.fireEvent('delete', record);
    },
    getProperty : function(id, property) {
        var index = this.getRecordIndex({ id : id});
        return this.data[index][property];
    },
    geNewId : function(){
        this.counterNewIds++;
        return 'new_' + this.counterNewIds;
    },
    updateProperty : function(id, property, value){
        var index = this.getRecordIndex({ id : id });
        if(index==-1){
            index = this.getRecordIndex(this.getNumeric(id));
        }
        if(index>=0){
            this.data[index][property] = value;
        }
        if(property == 'id'){
            this.recordPointers[value] = index;
        }
    },
    getNumeric : function(id){
        return id.replace(/[^0-9]/g,'');
    }
});

var WPModel = new DG.WeekPlannerModel();

var newAppointmentCounter = -1;
var moveAppointmentCounter = -1;
var resizeAppointmentCounter = -1;
var resizeAppointmentInitHeight = false;

var dragCoords = {
    el : {
        x : null,
        y: null
    },
    mouse : {
        x : null,
        y : null
    }
}

var els = {
    container : null,
    eventsContainer : null,
    appointments : null,
    currentAppointment : {
        container : null,   // Reference to element currently active, i.e. with blue header;
        el : null,
        contentEl : null,
        timeEl : null
    }
}

var elWidth;

var appointmentsOffsetTop = false;
var appointmentsOffsetLeft = false;

var currentZIndex = 20000;

var dayPositionArray = new Array();
var dayDateArray = new Array();

var dateStartOfWeek = false;
var newAppointmentWidth = false;

var startIdOfNewItems = 500000000;
var contentEditInProgress = false;
var toggleViewCounter = -1;
var objectToToggle = false;
var currentEditableTextArea = false;

var opera = navigator.userAgent.toLowerCase().indexOf('opera')>=0;

function editFromPopup(e,inputDiv){
	if(!inputDiv)inputDiv = this;
	if(!remote.urls.edit)return;
	if(inputDiv.id.indexOf('new_')>=0)return;

	var editEvent = window.open(remote.urls.edit + '?id=' + inputDiv.id,'editEvent','width=500,height=500,status=no');
	editEvent.focus();
}

function highlightAppointment(e,inputDiv){
	if(!inputDiv)inputDiv = this;
	var subDivs = inputDiv.getElements('DIV.weekScheduler_appointment_header');
	for(var i=0;i<subDivs.length;i++){
        subDivs[i].className = 'weekScheduler_appointment_headerActive';
	}
	if(els.currentAppointment.container && els.currentAppointment.container!=inputDiv){
		lowlightAppointment(els.currentAppointment.container);
	}
	els.currentAppointment.container = inputDiv;
}

function lowlightAppointment(inputDiv){
	var subDivs = inputDiv.getElements('DIV.weekScheduler_appointment_headerActive');
    var countDivs = subDivs.length;
	for(var i=0;i<countDivs;i++){
        subDivs[i].className = 'weekScheduler_appointment_header';
	}
}


/* updating content - this function is called from popup window */
function setElement_txt(id,text) {
	var ta = $(id).getElements('TEXTAREA')[0]
	ta.value = text;
	moveTextFromTextAreaField(false,ta);
}
// update bg color - this function is called from popup window */
function setElement_color(id,color) {
	$(id).setStyle('background-color', color);
    WPModel.updateProperty(id, 'bgColor', color);
}


function createNewAppointmentObjects(items) {
    if(!remote.useJSON){
        var items = items.split(/<item>/g);
    }

	for(var i=0;i<items.length;i++){
		var newItem = {};
        if(remote.useJSON){
            newItem = items[i];
            newItem.id = newItem.id + '';
            newItem.startDate = new Date(newItem.startDate);
            newItem.endDate = new Date(newItem.endDate);
        }else{
            var lines = items[i].split(/\n/g);
            for(var j=0;j<lines.length;j++){
                var key = lines[j].replace(/<([^>]+)>.*/g,'$1');
                if(key)key = key.trim();
                var pattern = new RegExp("<\/?" + key + ">","g");
                var value = lines[j].replace(pattern,'');
                value = value.trim();
                if(key=='startDate' || key=='endDate'){
                    var d = new Date(value);
                    value = d;
                }
                newItem[key] = value;
            }
        }

		if(newItem.id){
			var dayDiff = getDayOffsetFromStartOfWeek(newItem.startDate.getTime());

			topPos = getYPositionFromTime(newItem.startDate.getHours(),newItem.startDate.getMinutes());

			var elHeight = (newItem.endDate.getTime() - newItem.startDate.getTime()) / (60 * 60 * 1000);
			elHeight = Math.round((elHeight * (itemRowHeight + 1)) - 2);

            els.currentAppointment.el = createDomForAppointment({
                    x : (dayPositionArray[dayDiff] - appointmentsOffsetLeft),
                    y : topPos,
                    width : (newAppointmentWidth-(appointmentMarginSize*2)),
                    height : elHeight,
                    dataObj : newItem
            });

			els.currentAppointment.el.id = 'event' + newItem.id;
            setNewZIndex(els.currentAppointment.el);
			els.currentAppointment.timeEl = getCurrentTimeDiv(els.currentAppointment.el);
			els.currentAppointment.timeEl.setStyle('display','block');

			if(newItem.bgColor && newItem.bgColor.match(/^#[0-9A-F]{6}$/)){
				els.currentAppointment.el.setStyle('background-color', newItem.bgColor);
			}

			els.currentAppointment.contentEl  = getCurrentContentEl(els.currentAppointment.el);
			els.currentAppointment.contentEl.setStyle('height', (elHeight-20));

			setNewDOMAppointmentTime();
			autoResizeAppointment();

			els.currentAppointment.timeEl = null;

            var record = {
                'id'  : newItem.id,
                'description': newItem.description,
                'bgColor' : newItem.bgColor,
                'startDate' : newItem.startDate,
                'endDate' : newItem.endDate,
                'readOnly' : newItem.readOnly ? true : false
            };
            WPModel.addRecord(record);
            els.currentAppointment.el = null;
		}
	}
}

function setNewDOMAppointmentTime(){
    els.currentAppointment.timeEl.innerHTML = '<span>' + getCalculatedTimeFromDOMPosition(els.currentAppointment.el) + '</span>';
}

function getCalculatedTimeFromDOMPosition(inputObj) {
	var startTime = (getTopPos(inputObj) - appointmentsOffsetTop) / (itemRowHeight+1) + weekplannerStartHour;
	if(startTime>23)startTime = startTime - 24;

	var startHour = Math.floor(startTime);
	var hourPrefix = startHour < 10 ? '0' : '';

    var startMinute = Math.floor((startTime - startHour) *60);
	var startMinutePrefix = startMinute<10 ? '0' : '';

	var endTime = (getTopPos(inputObj) + inputObj.offsetHeight - appointmentsOffsetTop) / (itemRowHeight+1) + weekplannerStartHour;
	if(endTime>24)endTime = endTime - 24;

    var endHour = Math.floor(endTime);
	var endHourPrefix = endHour<10 ? '0' : '';

	var endMinute = Math.floor((endTime - endHour) *60);
	var endMinutePrefix = endMinute<10 ? '0' : '';

	return hourPrefix + startHour + ':' + startMinutePrefix + "" + startMinute + '-' + endHourPrefix + endHour + ':' + endMinutePrefix + "" +  endMinute;
}


function getDayOffsetFromStartOfWeek(timestamp){
    var ret = timestamp - dateStartOfWeek.getTime();
    return Math.floor(ret / (1000*60*60*24));
}

function setNewZIndex(div){
    currentZIndex = currentZIndex + 1;
    div.setStyle('z-index', currentZIndex);
}

/* Update date and hour properties for an appointment after move or drag */

function updateAppointmentRecord(id) {
	var obj = $(id);

	var timeArray = getTimeFromElementPosition(obj);
	var startDate = getDateFromAppointmentDomPosition(obj);
	var endDate = new Date();

	startDate.setHours(timeArray.start.hour);
	startDate.setMinutes(timeArray.start.minute);

    endDate.setTime(startDate.getTime());
	endDate.setHours(timeArray.end.hour);
	endDate.setMinutes(timeArray.end.minute);

    var record = {
        startDate : startDate,
        endDate : endDate,
        id : id,
    };
    WPModel.update(record);

	if(instantSave && WPModel.getProperty(id, 'description').length>0){
		saveAnItemToServer(id);
	}
}

function getNumeric(id){
    return id.replace(/[^0-9]/g,'');
}

function getYPositionFromTime(hour,minute){
	return Math.floor((hour - weekplannerStartHour) * (itemRowHeight+1) + (minute/60 * (itemRowHeight+1)));
}

function getItemsFromServer()
{
    var req = new Request({
        url: remote.urls.get,
        method : remote.method,
        noCache : 1,
        data: {
            getItems : 1,
            year:dateStartOfWeek.getFullYear(),
            month: (dateStartOfWeek.getMonth()/1+1),
            day : dateStartOfWeek.getDate(),
            json: remote.useJSON ? 1 : 0
        },
        onComplete : function(response) {
            clearAppointments();
            if(remote.useJSON){
                response = JSON.decode(response);
                if(!response.success && txt.loadError){
                    alert(txt.loadError);
                }
                response = response.data;
            }
            createNewAppointmentObjects(response);
        }
    });
    req.send();
}

function getCurrentTimeDiv(inputObj){
	return inputObj.getElements('DIV.weekScheduler_appointment_time')[0];
}

function getCurrentContentEl(inputDiv){
	return inputDiv.getElements('DIV.weekScheduler_appointment_txt')[0];
}

function getDateFromAppointmentDomPosition(inputObj){
	var leftPos = getLeftPos(inputObj);
	var d = new Date();
	var tmpTime = dateStartOfWeek.getTime();
	tmpTime = tmpTime + (1000*60*60*24 * Math.floor((leftPos-appointmentsOffsetLeft) / (dayPositionArray[1] - dayPositionArray[0])));
	d.setTime(tmpTime);
	return d;
}

function getTimeFromElementPosition(inputObj) {
	var startTime = (getTopPos(inputObj) - appointmentsOffsetTop) / (itemRowHeight+1) + weekplannerStartHour;
	if(startTime>23)startTime = startTime - 24;
	var startHour = Math.floor(startTime);
	var startMinute = Math.floor((startTime - startHour) *60);
	var endTime = (getTopPos(inputObj) + inputObj.offsetHeight - appointmentsOffsetTop) / (itemRowHeight+1) + weekplannerStartHour;
	if(endTime>24)endTime = endTime - 24;
	var endHour = Math.floor(endTime);
	var endMinute = Math.floor((endTime - endHour) *60);
	return {
        'start' : {
            'hour' : startHour,
            'minute' : startMinute
        },
        'end' : {
            'hour' : endHour,
            'minute' : endMinute
        }
    };
}

function initNewAppointment(e,inputObj) {
	if(document.all)e = event;
	if(!inputObj)inputObj = this;
	newAppointmentCounter = 0;

    dragCoords = {
        el : {
            x : getLeftPos(inputObj),
            y : getTopPos(inputObj)
        },
        mouse : {
            x : e.clientX,
            y : e.clientY
        }
    };

    elWidth = inputObj.offsetWidth;
	timerNewAppointment();

	return false;
}

function timerNewAppointment() {
	if(newAppointmentCounter>=0 && newAppointmentCounter<10){
		newAppointmentCounter = newAppointmentCounter + 1;
		setTimeout('timerNewAppointment()',30);
		return;
	}
	if(newAppointmentCounter==10){

		if(initMinutes){
			var topPos = dragCoords.mouse.y - appointmentsOffsetTop + document.documentElement.scrollTop + $('weekScheduler_content').scrollTop;
			topPos = topPos - (getCalculatedMinutesByYCoord(topPos) % initMinutes);
			var rest = (getCalculatedMinutesByYCoord(topPos) % initMinutes);
			if(rest!=0){
				topPos = topPos - (getCalculatedMinutesByYCoord(topPos) % initMinutes);
			}
		}else{
			var topPos = (dragCoords.el.y - appointmentsOffsetTop);
		}
        els.currentAppointment.el = createDomForAppointment({
                x : (dragCoords.el.x - appointmentsOffsetLeft),
                y : topPos,
                width : (elWidth-(appointmentMarginSize*2)),
                height : null,
                dataObj : { description : '', readOnly : false }
        });
		els.currentAppointment.el.id = 'new_' + startIdOfNewItems;

        var record = {
            id : els.currentAppointment.el.id,
            description : '',
            startDate : null,
            endDate : null,
            bgColor : '#FFFFFF'
        };
        WPModel.addRecord(record);

		startIdOfNewItems++;
		els.currentAppointment.contentEl  = getCurrentContentEl(els.currentAppointment.el);

        setNewZIndex(els.currentAppointment.el);

        els.currentAppointment.el.setStyles({
            'height' : 20
        });
		els.currentAppointment.timeEl = getCurrentTimeDiv(els.currentAppointment.el);
		els.currentAppointment.timeEl.setStyle('display', 'block');

	}
}

function initResizeAppointment(e) {
	if(document.all)e = event;
	els.currentAppointment.el = this.parentNode;
	els.currentAppointment.contentEl  = getCurrentContentEl(els.currentAppointment.el);
    setNewZIndex(els.currentAppointment.el);
	resizeAppointmentCounter = 0;
    dragCoords = {
        el : {
            x : getLeftPos(els.currentAppointment.el),
            y : getTopPos(els.currentAppointment.el)
        },
        mouse : {
            x : e.clientX,
            y : e.clientY
        }
    }

	resizeAppointmentInitHeight = els.currentAppointment.el.style.height.replace('px','')/1;

	activateResize.delay(100);
	return false;
}
function activateResize(){
    if(resizeAppointmentCounter == 0){
        resizeAppointmentCounter = 10;
		els.currentAppointment.timeEl = getCurrentTimeDiv(els.currentAppointment.el);
		els.currentAppointment.timeEl.style.display='block';
    }
}

function initMoveAppointment(e,inputObj) {
	if(document.all)e = event;
	if(!inputObj)inputObj = this.parentNode;

	els.currentAppointment.el = inputObj;
	els.currentAppointment.contentEl  = getCurrentContentEl(els.currentAppointment.el);
    setNewZIndex(els.currentAppointment.el);
	moveAppointmentCounter = 0;
	dragCoords.el.x = getLeftPos(inputObj);
	dragCoords.el.y = getTopPos(inputObj);
	elWidth = inputObj.offsetWidth;

	dragCoords.mouse.x = e.clientX;
	dragCoords.mouse.y = e.clientY;

	activateMove.delay(200);
	return false;
}

function activateMove(){
    if(moveAppointmentCounter == 0){
        moveAppointmentCounter = 10;
		els.currentAppointment.timeEl = getCurrentTimeDiv(els.currentAppointment.el);
		els.currentAppointment.timeEl.style.display='block';
    }
}

function isReadOnly(el){
    return WPModel.getProperty(el.id, 'readOnly');
}

function getCalculatedMinutesByYCoord(topPos) {
	var time = (topPos) / (itemRowHeight+1);

	var hour = Math.floor(time);
	var minute = Math.floor((time - hour) *60);
	return minute;
}

function schedulerMouseMove(e) {
	if(document.all)e = event;
	if(newAppointmentCounter==10){
		if(!els.currentAppointment.el)return;
		var tmpHeight = e.clientY - dragCoords.mouse.y;
		els.currentAppointment.el.style.height = Math.max(20,tmpHeight) + 'px';
        setNewDOMAppointmentTime();
	}

	if(moveAppointmentCounter==10){
		var topPos = (e.clientY - dragCoords.mouse.y + dragCoords.el.y - appointmentsOffsetTop);
		els.currentAppointment.el.style.top = topPos + 'px';
		var destinationLeftPos = false;
		for(var no=0;no<dayPositionArray.length;no++){
			if(e.clientX>dayPositionArray[no])destinationLeftPos = dayPositionArray[no];
		}
		els.currentAppointment.el.style.left = (destinationLeftPos + appointmentMarginSize -2) + 'px';
        if(els.currentAppointment.timeEl){
		    setNewDOMAppointmentTime();
        }
	}

	if(resizeAppointmentCounter==10){
		els.currentAppointment.contentEl.setStyle('height', Math.max((resizeAppointmentInitHeight + e.clientY - dragCoords.mouse.y),10)-8);
		els.currentAppointment.el.setStyle('height', Math.max((resizeAppointmentInitHeight + e.clientY - dragCoords.mouse.y),10));
		setNewDOMAppointmentTime();
	}

}

function repositionFooter(inputDiv) {
	var el = inputDiv.getElements('DIV.weekScheduler_appointment_footer')[0];
    el.setStyle('bottom', '-1');
}

function moveTextFromTextAreaField(e,inputObj,discardContentUpdate){
	if(!inputObj)inputObj = this;
	inputObj.setStyle('display','none');
	var spans = inputObj.parentNode.getElements('DIV');
	for(var no=0;no<spans.length;no++){
		if(spans[no].className=='weekScheduler_appointment_txt'){
			if(!discardContentUpdate){
                WPModel.updateProperty(inputObj.parentNode.id, 'description', inputObj.value);
				spans[no].innerHTML = '<span style="float:right;">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp&nbsp;&nbsp;&nbsp;</span>' + inputObj.value.replace(/\n/g,'<br>');
			}
		}
        if(spans[no].hasClass('weekScheduler_appointment_txt') || spans[no].hasClass('weekScheduler_appointment_footer') || spans[no].hasClass('weekScheduler_appointment_time')){
            spans[no].setStyle('display','block');
        }
	}
	contentEditInProgress = false;
	currentEditableTextArea = false;
	repositionFooter(inputObj.parentNode);
	if(instantSave && WPModel.getProperty(inputObj.parentNode.id, 'description').length>0){
		saveAnItemToServer(inputObj.parentNode.id);
	}
}

/*
This function clears all appointments from the screen - Used when switching from one week to another
*/
function clearAppointments() {
    var els = $(document.body).getElements('.weekScheduler_anAppointment');
    var countEls = els.length;
    for(var i=0;i<countEls;i++){
        els[i].destroy();
    }
    WPModel.clear();
}

function saveAnItemToServer(inputId){
    var record = WPModel.getRecord(inputId);
	if(!record.description)record.description='';
	if(!record.bgColor)record.bgColor='';
	if(!record.startDate)updateAppointmentRecord(inputId);;

    var req = new Request({
        url: remote.urls.save,
        method : remote.method,
        noCache : 1,
        data: {
            saveAnItem:true,
            id: record['id'],
            description : record['description'],
            bgColor : record['bgColor'],
            startDate : record['startDate'].toGMTString().replace('UTC','GMT'),
            endDate : record['endDate'].toGMTString().replace('UTC','GMT'),
            readOnly : record['readOnly'] ? '1' : '0',
            newItem : record['id'].indexOf('new_')>=0 ? '1' : '0',
            json : remote.useJSON
        },
        onComplete : function(response) {
            if(remote.useJSON){
                var json = JSON.decode(response);
                if(json.success){
                   var newId = json.data.id +'';
                }else{
                    alert('Could not confirm that data was saved successfully');
                    return;
                }
            }else{
                var newId = response;
            }
            var oldId = record['id'];

            if(oldId.indexOf('new_')>=0){
                var domObj = $(oldId);
                domObj.id = 'event' + newId;
                WPModel.updateProperty(oldId, 'id', newId);
                if(!inlineTextAreaEnabled){
                    editFromPopup(false,domObj);
                }
            }
        }
    });
    req.send();
}

function ffEndEdit(e) {
	if(!currentEditableTextArea)return;
	if (e.target) source = e.target;
		else if (e.srcElement) source = e.srcElement;
		if (source.nodeType == 3) // defeat Safari bug
			source = source.parentNode;
	if(source.tagName.toLowerCase()!='textarea')currentEditableTextArea.blur();
}

function mouseDownOnAppointment(e) {
    var source = e.target;
	if(!source.hasClass('weekScheduler_appointment_txt') && !source.hasClass('weekScheduler_anAppointment')){
		return;
	}
	toggleViewCounter = 0;
	objectToToggle = this;
	timerToggleView();
}

function timerToggleView() {
	if(toggleViewCounter>=0 && toggleViewCounter<10){
		toggleViewCounter = toggleViewCounter + 1;
		setTimeout('timerToggleView()',50);
	}
	if(toggleViewCounter==10){
		toggleViewCounter = -1;
		toggleAppointmentView(false,objectToToggle);
	}
}

function toggleAppointmentView(e,inputObj) {
	if(!inlineTextAreaEnabled)return;
	els.currentAppointment.el = inputObj;
	var spans = inputObj.getElements('DIV');

	var tmpValue = "";
	for(var no=0;no<spans.length;no++){
		if(spans[no].className=='weekScheduler_appointment_txt'){

			tmpValue = WPModel.getProperty(inputObj.id, 'description');
		}
		if(spans[no].className=='weekScheduler_appointment_txt' || spans[no].className=='weekScheduler_appointment_footer' || spans[no].className=='weekScheduler_appointment_time'){
			spans[no].style.display='none';
		}
	}

	var ta = els.currentAppointment.el.getElements('TEXTAREA')[0];
    ta.setStyles({
        'width' : (els.currentAppointment.el.clientWidth - 6),
        'height' : (els.currentAppointment.el.offsetHeight-14),
        'display' : 'inline'

    });
	ta.value = tmpValue;
	contentEditInProgress = true;
	currentEditableTextArea = ta;
	ta.focus();
}

/* Checking keyboard event for the textarea */

function keyboardEventTextarea(e)
{
	if(document.all)e = event;
	if(e.keyCode==27){	// Escape key
		moveTextFromTextAreaField(false,this,true);
	}
}

function createDomForAppointment(obj)
{
    var appCss = {
        left: obj.x,
        top: obj.y,
        width : obj.width,
        height : obj.height ? obj.height : null,
        'background-color' : obj.dataObj.bgColor ? obj.dataObj.bgColor : '#FFFFFF'
    };
	var appDiv = getNewAppointmentDomContainer(appCss, obj.dataObj);

	addNewAppointmentTimeElement(appDiv);
	addNewAppointmentHeader(appDiv, obj.dataObj);
	addNewAppointmentContentDom(appDiv, obj.dataObj.description);
	addNewAppointmentTextAreaField(appDiv);
    addNewAppointmentColorCodeElement(appDiv);
    addNewAppointmentFooter(appDiv, obj.dataObj);

	els.eventsContainer.adopt(appDiv);
	return appDiv;
}

function getNewAppointmentDomContainer(css, dataObj){
	var div = new Element('DIV');
    div.onclick = highlightAppointment;
    if(!dataObj.readOnly){
        div.ondblclick = editFromPopup;
        div.addEvent('mousedown', mouseDownOnAppointment);
        if(inlineTextAreaEnabled){
            div.title = txt.holdDownToEdit;
        }
    }
	div.className='weekScheduler_anAppointment';
    div.setStyles(css);

    return div
}

function addNewAppointmentColorCodeElement(parent){
	var colorCodeDiv = new Element('DIV');
	colorCodeDiv.addClass('weekScheduler_appointment_colorCodes');
	parent.adopt(colorCodeDiv);

}

function addNewAppointmentTimeElement(parent){
	var timeDiv =new Element('DIV');
	timeDiv.className='weekScheduler_appointment_time';
	timeDiv.innerHTML = '<span></span>';
    parent.adopt(timeDiv);
}

function addNewAppointmentHeader(parent, dataObj){
	var header =new Element('DIV');
	header.addClass('weekScheduler_appointment_header');
	header.innerHTML = '<span></span>';
    if(!dataObj.readOnly){
        header.onmousedown = initMoveAppointment;
        header.style.cursor = 'move';
    }
    parent.adopt(header);
}

function addNewAppointmentContentDom(parent, initialHtml){
	var span = new Element('DIV');
	var innerSpan = new Element('SPAN');
	innerSpan.innerHTML = '<span style="float:right;">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp&nbsp;&nbsp;&nbsp;</span>' + initialHtml;
	span.adopt(innerSpan);
	span.addClass('weekScheduler_appointment_txt');
	parent.adopt(span);

}
function addNewAppointmentTextAreaField(parent){
	var textarea =new Element('TEXTAREA');
	textarea.className='weekScheduler_appointment_textarea';
	textarea.style.display='none';
	textarea.onblur = moveTextFromTextAreaField;
	textarea.onkeyup = keyboardEventTextarea;
    parent.adopt(textarea);
}

function addNewAppointmentFooter(parent, dataObj){
	var footerDiv = new Element('DIV');
	footerDiv.addClass('weekScheduler_appointment_footer');

	footerDiv.innerHTML = '<span></span>';
    if(!dataObj.readOnly){
        footerDiv.setStyle('cursor', 'n-resize');
	    footerDiv.onmousedown = initResizeAppointment;
    }
    parent.adopt(footerDiv);
}

function executeMouseUpEvents() {

    if(newAppointmentCounter==10){
        if(!els.currentAppointment.el)return;
        if(inlineTextAreaEnabled){
            var spans = els.currentAppointment.el.getElements('DIV');
            for(var no=0;no<spans.length;no++){
                if(spans[no].className=='weekScheduler_appointment_txt' || spans[no].className=='weekScheduler_appointment_footer' || spans[no].className=='weekScheduler_appointment_time'){
                    spans[no].style.display='none';
                }
            }

            var ta = els.currentAppointment.el.getElements('TEXTAREA')[0];
            ta.setStyles({
                width : (els.currentAppointment.el.clientWidth - 6),
                height : (els.currentAppointment.el.offsetHeight-14),
                display : 'inline'
            });
            ta.focus();
        }
        else{
            saveAnItemToServer(els.currentAppointment.el.id);
        }
    }
	
	if(snapToMinutes && els.currentAppointment.el && moveAppointmentCounter==10){
		topPos = getTopPos(els.currentAppointment.el) - appointmentsOffsetTop;

		var minute = getCalculatedMinutesByYCoord(topPos);
		var rest = (minute % snapToMinutes);
		if(rest>(snapToMinutes/2)){
			topPos = topPos + (snapToMinutes/60*(itemRowHeight+1)) - ((rest/60)*(itemRowHeight+1));
		}else{
			topPos = topPos - ((rest/60)*(itemRowHeight+1));
		}
		var minute = getCalculatedMinutesByYCoord(topPos);
		var rest = (minute % snapToMinutes);
		if(rest!=0){
			topPos = topPos - ((rest/60)*(itemRowHeight+1));
		}

		var minute = getCalculatedMinutesByYCoord(topPos);
		var rest = (minute % snapToMinutes);
		if(rest!=0){
			topPos = topPos - ((rest/60)*(itemRowHeight+1));
		}
		els.currentAppointment.el.style.top = topPos + 'px';
		els.currentAppointment.timeEl.innerHTML = '<span>' + getCalculatedTimeFromDOMPosition(els.currentAppointment.el) + '</span>';
	}

	if(els.currentAppointment.el && snapToMinutes && (resizeAppointmentCounter==10 || newAppointmentCounter)){
		autoResizeAppointment();
	}


	if(els.currentAppointment.el && !contentEditInProgress){
		repositionFooter(els.currentAppointment.el);
		updateAppointmentRecord(els.currentAppointment.el.id);
	}

	els.currentAppointment.el = false;
	els.currentAppointment.timeEl = false;
	moveAppointmentCounter = -1;
	resizeAppointmentCounter = -1;
	newAppointmentCounter = -1;
	toggleViewCounter = -1;

}

function autoResizeAppointment() {
	var tmpPos = getTopPos(els.currentAppointment.el) - appointmentsOffsetTop + els.currentAppointment.el.offsetHeight;
	var startPos = tmpPos;

	var minute = getCalculatedMinutesByYCoord(tmpPos);

	var rest = (minute % snapToMinutes);
	var height = els.currentAppointment.el.style.height.replace('px','')/1;

	if(rest>(snapToMinutes/2)){
		tmpPos = tmpPos + snapToMinutes - (minute % snapToMinutes);
	}else{
		tmpPos = tmpPos - (minute % snapToMinutes);
	}

	var minute = getCalculatedMinutesByYCoord(tmpPos);
	if((minute % snapToMinutes)!=0){
		tmpPos = tmpPos - (minute % snapToMinutes);
	}
	var minute = getCalculatedMinutesByYCoord(tmpPos);
	if((minute % snapToMinutes)!=0){
		tmpPos = tmpPos - (minute % snapToMinutes);
	}

	els.currentAppointment.el.setStyle('height', height + tmpPos - startPos);
	els.currentAppointment.timeEl.innerHTML = '<span>' + getCalculatedTimeFromDOMPosition(els.currentAppointment.el) + '</span>';

}

function schedulerKeyboardEvent(e){
	if(document.all)e = event;
	if(e.keyCode==46 && els.currentAppointment.container){
        var readOnly = WPModel.getProperty(els.currentAppointment.container.id, 'readOnly');

		if(!readOnly && confirm(txt.delete)){

            var req = new Request({
                url: remote.urls.delete,
                method : remote.method,
                noCache : 1,
                data: {
                    deleteItem: true,
                    id : els.currentAppointment.container.id,
                    json : remote.useJSON
                },
                onComplete : function(response) {
                    if(remote.useJSON){
                        response = JSON.decode(response);
                        success = response.success;
                    }else{
                        success = response;
                    }
                    if(success){
                        els.currentAppointment.container.parentNode.removeChild(els.currentAppointment.container);
                        els.currentAppointment.container = false;
                    }else{
                        // Error handling - event not deleted
                        alert('Could not confirm that event has been deleted. Make sure that the script is configured correctly');
                    }
                }
            });
            req.send();
		}
	}
}

function getTopPos(inputObj) {
  var returnValue = inputObj.offsetTop;
  while((inputObj = inputObj.offsetParent) != null){
  	if(inputObj.tagName!='HTML')returnValue += inputObj.offsetTop;
  }
  return returnValue;
}

function getLeftPos(inputObj) {
  var returnValue = inputObj.offsetLeft;
  while((inputObj = inputObj.offsetParent) != null){
  	if(inputObj.tagName!='HTML')returnValue += inputObj.offsetLeft;
  }
  return returnValue;
}


function cancelSelectionEvent(e) {
	if(document.all)e = event;
	if (e.target) source = e.target;
		else if (e.srcElement) source = e.srcElement;
		if (source.nodeType == 3) // defeat Safari bug
			source = source.parentNode;
	if(source.tagName.toLowerCase()=='input' || source.tagName.toLowerCase()=='textarea')return true;
	return false;
}
function initWeekScheduler()
{
	els.container = $('weekScheduler_container');
	if(!document.all)els.container.onclick = ffEndEdit;
	els.eventsContainer = $('weekScheduler_appointments');
	var subDivs = els.eventsContainer.getElements('DIV');
	for(var no=0;no<subDivs.length;no++){
		if(subDivs[no].className=='weekScheduler_appointmentHour'){
			subDivs[no].onmousedown = initNewAppointment;
			if(!newAppointmentWidth)newAppointmentWidth = subDivs[no].offsetWidth;
		}
		if(subDivs[no].className=='weekScheduler_appointments_day'){
			dayPositionArray[dayPositionArray.length] = getLeftPos(subDivs[no]);
		}

	}
	if(initialScrollToHour > weekplannerStartHour)$('weekScheduler_content').scrollTop = ((initialScrollToHour - weekplannerStartHour)*(itemRowHeight+1));

	//	initialScrollToHour
	appointmentsOffsetTop = getTopPos(els.eventsContainer);
	appointmentsOffsetLeft = 2 - appointmentMarginSize;

	document.documentElement.onmousemove = schedulerMouseMove;
	document.documentElement.onselectstart = cancelSelectionEvent;
	document.documentElement.onmouseup = executeMouseUpEvents;
	document.documentElement.onkeydown = schedulerKeyboardEvent;

	var tmpDate = new Date();
	var dateItems = initDateToShow.split(/\-/g);
	tmpDate.setFullYear(dateItems[0]);
	tmpDate.setDate(dateItems[2]/1);
	tmpDate.setMonth(dateItems[1]/1-1);
	tmpDate.setHours(1);
	tmpDate.setMinutes(0);
	tmpDate.setSeconds(0);

	var day = tmpDate.getDay();
	if(day==0)day=7;
	if(day>1){
		var time = tmpDate.getTime();
		time = time - (1000*60*60*24) * (day-1);
		tmpDate.setTime(time);
	}
	dateStartOfWeek = new Date(tmpDate);

	updateHeaderDates();

	if(remote.urls.get){
		getItemsFromServer();
        if(remote.refreshViewAfterSeconds > 0){
            executePeriodicalViewUpdate.periodical(remote.refreshViewAfterSeconds * 1000);
        }
	}
}

function executePeriodicalViewUpdate(){
    if(!isInUserActionMode()){
        refreshView();
    }
}

function isInUserActionMode(){
    return (moveAppointmentCounter >=0 || resizeAppointmentCounter>=0 || newAppointmentCounter>=0 || toggleViewCounter>=0);
}

function displayPreviousWeek() {
	offsetStartTimeByWeeks(-1);
	refreshView();
}

function displayNextWeek(){
	offsetStartTimeByWeeks(1);
	refreshView();
}

function offsetStartTimeByWeeks(weekOffset){
	var tmpTime = dateStartOfWeek.getTime();
	tmpTime = tmpTime + (1000*60*60*24*7*weekOffset);
	dateStartOfWeek.setTime(tmpTime);
}

function refreshView(){
	updateHeaderDates();

	getItemsFromServer();
}

function updateHeaderDates() {
	var weekScheduler_dayRow = $('weekScheduler_dayRow');
	var subDivs = weekScheduler_dayRow.getElements('DIV');
	var tmpDate2 = new Date(dateStartOfWeek);

    var oneDay = (1000*60*60*24);

	for(var no=0;no<subDivs.length;no++){
		var month = tmpDate2.getMonth()/1 + 1;
		var date = tmpDate2.getDate();
		var tmpHeaderFormat = " " + headerDateFormat;
		tmpHeaderFormat = tmpHeaderFormat.replace('d',date);
		tmpHeaderFormat = tmpHeaderFormat.replace('m',month);

		subDivs[no].getElements('SPAN')[0].innerHTML = tmpHeaderFormat;

		dayDateArray[no] = month + '|' + date;

		var timestamp = tmpDate2.getTime();
		timestamp += oneDay;
		tmpDate2.setTime(timestamp);
	}
}
window.addEvent('domready', function() {
    initWeekScheduler();
});

